/*
 * Gtk-IPtables is Copyright (C) 2006  Daniel E. Testa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "create_rule_win.h"

/* create rule window widgets */
typedef struct _st_wnd
{
  GtkWidget *create_wnd;
  GtkWidget *cb_tables;
  GtkWidget *lbl_tables;
  GtkWidget *cb_chains;
  GtkWidget *lbl_chains;
  GtkWidget *cb_protocol;
  GtkWidget *lbl_protocol;
  GtkWidget *bt_tcp;
  GtkWidget *bt_icmp;
  GtkWidget *cb_jump;
  GtkWidget *lbl_src_port;
  GtkWidget *lbl_src_address;
  GtkWidget *ed_src_address;
  GtkWidget *ed_src_port;
  GtkWidget *lbl_dest_address;
  GtkWidget *ed_dest_address;
  GtkWidget *lbl_dest_port;
  GtkWidget *ed_dest_port;
  GtkWidget *lbl_jump;
  GtkWidget *bt_set_reject;
  GtkWidget *bt_log_rule;
  GtkWidget *bt_accept;
  GtkWidget *bt_cancel;
  GtkWidget *chk_in_inter;
  GtkTooltips *tt_in_inter;
  GtkTooltips *tt_out_inter;
  GtkWidget *ed_in_inter;
  GtkWidget *ed_out_inter;
  GtkWidget *chk_out_inter;
  GtkWidget *chk_fragment;
  GtkWidget *chk_frag_w_options;
  GtkWidget *chk_set_counters;
  GtkWidget *ed_packages;
  GtkWidget *ed_bytes;
} STWND_CRT;

/* 
 * window controls structure is long enought to contains
 * containers also. it's not the best design but...
 */
typedef struct _st_containers
{
  GtkWidget *frm_chain_sel;
  GtkWidget *frm_protocols;
  GtkWidget *frm_src_addr_port;
  GtkWidget *frm_dest_addr_port;
  GtkWidget *frm_actions;
  GtkWidget *frm_inter;
  GtkWidget *frm_options;
  GtkWidget *bx_tables_chains;
  GtkWidget *bx_main;
  GtkWidget *bx_protocols;
  GtkWidget *bx_src_opts;
  GtkWidget *bx_dest_opts;
  GtkWidget *bx_actions;
  GtkWidget *bx_win_cmds;
  GtkWidget *bx_interfaces;
  GtkWidget *bx_options;
  GtkWidget *bx_protocols_aux;
  GtkWidget *bx_jumps;
} STWND_CTRS;


/* window structure */
static STWND_CRT main_wnd;

/* containers structure */
static STWND_CTRS main_ctrns;

extern void update_rules();
extern void set_tcp_win();
extern void set_icmp_win();
extern void set_reject_win();
extern void log_rule_win();

static void on_cb_tables_changed(GtkWidget *widget, gpointer data);

static void create_cb_tables()
{
  GList *gl_tables = NULL;

  main_wnd.lbl_tables = gtk_label_new("  To Table ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_tables_chains), 
		     main_wnd.lbl_tables, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_tables);

  gl_tables = g_list_append(gl_tables, TABLE_FILTER);
  gl_tables = g_list_append(gl_tables, TABLE_NAT);  
  gl_tables = g_list_append(gl_tables, TABLE_MANGLE);
  
  main_wnd.cb_tables = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(main_wnd.cb_tables), gl_tables);
  gtk_combo_set_value_in_list(GTK_COMBO(main_wnd.cb_tables), TRUE, FALSE);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(main_wnd.cb_tables)->entry), "changed",
		     GTK_SIGNAL_FUNC (on_cb_tables_changed), NULL);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_tables_chains), 
		     main_wnd.cb_tables, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.cb_tables);
}

static void create_cb_chains()
{
  GList *gl_chains = NULL;

  main_wnd.lbl_chains = gtk_label_new("      To chain ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_tables_chains), 
		     main_wnd.lbl_chains, FALSE, TRUE,0);
  gtk_widget_show(main_wnd.lbl_chains);

  gl_chains = g_list_append(gl_chains, CHAIN_SEP_FILTER);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_INPUT);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_FORWARD);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_OUTPUT);
  gl_chains = g_list_append(gl_chains, CHAIN_SEP_NAT);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_PREROUTING);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_POSTROUTING);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_OUTPUT);
  gl_chains = g_list_append(gl_chains, CHAIN_SEP_MANGLE);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_PREROUTING);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_OUTPUT);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_INPUT);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_FORWARD);
  gl_chains = g_list_append(gl_chains, CHAIN_ITM_POSTROUTING);  

  main_wnd.cb_chains = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(main_wnd.cb_chains), gl_chains);
  gtk_combo_set_value_in_list(GTK_COMBO(main_wnd.cb_chains), TRUE, FALSE);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(main_wnd.cb_chains)->entry), "changed",
		     GTK_SIGNAL_FUNC (on_cb_chains_changed), NULL);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_tables_chains), 
		     main_wnd.cb_chains, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.cb_chains);
}

static void create_cb_protocol()
{
  GList *gl_protocol = NULL;

  main_wnd.lbl_protocol = gtk_label_new("  Protocol ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_protocols_aux), 
		     main_wnd.lbl_protocol, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_protocol);

  gl_protocol = g_list_append(gl_protocol, PROTO_TCP);
  gl_protocol = g_list_append(gl_protocol, PROTO_UDP);
  gl_protocol = g_list_append(gl_protocol, PROTO_ICMP);

  main_wnd.cb_protocol = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(main_wnd.cb_protocol), gl_protocol);
  gtk_combo_set_value_in_list(GTK_COMBO(main_wnd.cb_protocol), TRUE, FALSE);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(main_wnd.cb_protocol)->entry), "changed",
		     GTK_SIGNAL_FUNC(on_cb_protocol_changed), NULL);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_protocols_aux), 
		     main_wnd.cb_protocol, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.cb_protocol);
}

static void create_bt_tcp()
{
  main_wnd.bt_tcp = gtk_button_new_with_label(" Set TCP Protocol ");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_protocols_aux), main_wnd.bt_tcp, FALSE, FALSE, 15);
  gtk_signal_connect(GTK_OBJECT(main_wnd.bt_tcp), "clicked",
		     GTK_SIGNAL_FUNC(on_bt_tcp_clicked), NULL);
  gtk_widget_show(main_wnd.bt_tcp);
}

static void create_bt_icmp()
{
  main_wnd.bt_icmp = gtk_button_new_with_label(" Set ICMP Protocol ");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_protocols_aux), main_wnd.bt_icmp, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(main_wnd.bt_icmp), "clicked",
		     GTK_SIGNAL_FUNC(on_bt_icmp_clicked), NULL);
  gtk_widget_set_sensitive(main_wnd.bt_icmp, FALSE);
  gtk_widget_show(main_wnd.bt_icmp);
}

static void create_src_port_ed()
{
  main_wnd.lbl_src_port = gtk_label_new("     Port ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_src_opts), main_wnd.lbl_src_port, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_src_port);

  main_wnd.ed_src_port = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_src_port), "changed",
		     GTK_SIGNAL_FUNC (sport_callback), main_wnd.ed_src_port);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_src_opts), main_wnd.ed_src_port, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_src_port);
}

static void create_src_address_ed()
{
  main_wnd.lbl_src_address = gtk_label_new("  Address ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_src_opts), main_wnd.lbl_src_address, 
		     FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_src_address);

  main_wnd.ed_src_address = gtk_entry_new();

  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_src_address), "changed",
		     GTK_SIGNAL_FUNC (saddr_callback), main_wnd.ed_src_address);

  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_src_opts), main_wnd.ed_src_address, 
		     FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_src_address);
}

static void create_dest_address_ed()
{
  main_wnd.lbl_dest_address = gtk_label_new("  Address ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_dest_opts), main_wnd.lbl_dest_address, 
		     FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_dest_address);

  main_wnd.ed_dest_address = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_dest_address), "changed",
		     GTK_SIGNAL_FUNC (daddr_callback), main_wnd.ed_dest_address);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_dest_opts), main_wnd.ed_dest_address, 
		     FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_dest_address);
}

static void create_dest_port_ed()
{
  main_wnd.lbl_dest_port = gtk_label_new("     Port ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_dest_opts), main_wnd.lbl_dest_port, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_dest_port);

  main_wnd.ed_dest_port = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_dest_port), "changed",
		     GTK_SIGNAL_FUNC (dport_callback), main_wnd.ed_dest_port);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_dest_opts), main_wnd.ed_dest_port, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_dest_port);
}

static void create_cb_jump()
{
  GList *gl_jump = NULL;

  main_wnd.lbl_jump = gtk_label_new("  Jump ");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_jumps), main_wnd.lbl_jump, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.lbl_jump);

  gl_jump = g_list_append(gl_jump, JUMP_ACCEPT);
  gl_jump = g_list_append(gl_jump, JUMP_DROP);
  gl_jump = g_list_append(gl_jump, JUMP_QUEUE);
  gl_jump = g_list_append(gl_jump, JUMP_RETURN);
  gl_jump = g_list_append(gl_jump, JUMP_REJECT);

  main_wnd.cb_jump = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(main_wnd.cb_jump), gl_jump);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(main_wnd.cb_jump)->entry), "changed",
		     GTK_SIGNAL_FUNC(jump_callback), NULL);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_jumps), main_wnd.cb_jump, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.cb_jump);
}

static void create_bt_set_reject()
{
  main_wnd.bt_set_reject = gtk_button_new_with_label(" Set REJECT Type ");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_jumps), main_wnd.bt_set_reject, FALSE, FALSE, 15);
  gtk_signal_connect(GTK_OBJECT(main_wnd.bt_set_reject), "clicked",
		     GTK_SIGNAL_FUNC(set_reject_win), NULL); 
  gtk_widget_set_sensitive(main_wnd.bt_set_reject, FALSE);
  gtk_widget_show(main_wnd.bt_set_reject);
}

static void create_bt_log_rule()
{
  main_wnd.bt_log_rule = gtk_button_new_with_label(" LOG This Rule ");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_jumps), main_wnd.bt_log_rule, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(main_wnd.bt_log_rule), "clicked",
		     GTK_SIGNAL_FUNC(log_rule_win), NULL); 
  gtk_widget_show(main_wnd.bt_log_rule);
}
  
static void create_bt_accept()
{
  GtkWidget *img_label;

  main_wnd.bt_accept = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_accept), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_accept_clicked), NULL);

  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_accept.png", " Accept ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_accept), img_label);
  gtk_box_pack_end(GTK_BOX (main_ctrns.bx_win_cmds), main_wnd.bt_accept, FALSE, FALSE, 0);
  gtk_widget_show(main_wnd.bt_accept);
}

static void create_bt_cancel()
{
  GtkWidget *img_label;
  
  main_wnd.bt_cancel = gtk_button_new();
  gtk_signal_connect_object(GTK_OBJECT (main_wnd.bt_cancel), "clicked",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy), 
			    GTK_OBJECT (main_wnd.create_wnd));
  gtk_box_pack_end(GTK_BOX (main_ctrns.bx_win_cmds), main_wnd.bt_cancel, FALSE, FALSE, 0);
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_cancel.png", " Cancel ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_cancel), img_label);
  gtk_widget_show(main_wnd.bt_cancel);
}

static void create_chk_in_inter()
{
  main_wnd.chk_in_inter = gtk_check_button_new_with_label("In-Inter");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_interfaces), main_wnd.chk_in_inter, FALSE, FALSE, 5);
  gtk_widget_show(main_wnd.chk_in_inter);

  main_wnd.tt_in_inter = gtk_tooltips_new();
  gtk_tooltips_set_tip(main_wnd.tt_in_inter, main_wnd.chk_in_inter,
		       "ONLY for INPUT, FORWARD and PREROUTING", NULL);

  main_wnd.ed_in_inter = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_in_inter), "changed",
		     GTK_SIGNAL_FUNC (ininter_callback), main_wnd.ed_in_inter);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_interfaces), main_wnd.ed_in_inter, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_in_inter);
}

static void create_chk_out_inter()
{
  main_wnd.chk_out_inter = gtk_check_button_new_with_label("Out-Inter");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_interfaces), main_wnd.chk_out_inter, FALSE, FALSE, 5);
  gtk_widget_show(main_wnd.chk_out_inter);

  main_wnd.tt_out_inter = gtk_tooltips_new();
  gtk_tooltips_set_tip(main_wnd.tt_out_inter, main_wnd.chk_out_inter,
		       "ONLY for OUTPUT, FORWARD and POSTROUTING", NULL);

  main_wnd.ed_out_inter = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_out_inter), "changed",
		     GTK_SIGNAL_FUNC (outinter_callback), main_wnd.ed_out_inter);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_interfaces), main_wnd.ed_out_inter, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.ed_out_inter);
}

/* Create the rest of the options checkboxes. */
static void create_options_chks()
{
  
  main_wnd.chk_fragment = gtk_check_button_new_with_label("Fragment");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_options), main_wnd.chk_fragment, FALSE, TRUE, 5);
  gtk_signal_connect(GTK_OBJECT (main_wnd.chk_fragment), "toggled",
		     GTK_SIGNAL_FUNC (on_chk_fragment_clicked), NULL);
  gtk_widget_show(main_wnd.chk_fragment);

  main_wnd.chk_frag_w_options = gtk_check_button_new_with_label("Fragment with ! Option");
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_options), main_wnd.chk_frag_w_options, 
		     FALSE, TRUE, 5);
  gtk_signal_connect(GTK_OBJECT (main_wnd.chk_frag_w_options), "toggled",
		     GTK_SIGNAL_FUNC (on_chk_frag_w_options_clicked), NULL);
  gtk_widget_show(main_wnd.chk_frag_w_options);

  main_wnd.chk_set_counters = gtk_check_button_new_with_label("Set Counters");
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_options), main_wnd.chk_set_counters, FALSE, TRUE, 5);
  gtk_widget_show(main_wnd.chk_set_counters);

  main_wnd.ed_packages = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_packages), "changed",
		     GTK_SIGNAL_FUNC(pkts_func), main_wnd.ed_packages);
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_options), main_wnd.ed_packages, FALSE, TRUE, 0);
  gtk_widget_set_usize(main_wnd.ed_packages, 65, 0);
  gtk_widget_show(main_wnd.ed_packages);

  main_wnd.ed_bytes = gtk_entry_new();
  gtk_signal_connect(GTK_OBJECT(main_wnd.ed_bytes), "changed",
		     GTK_SIGNAL_FUNC(bytes_func), main_wnd.ed_bytes);
  gtk_box_pack_start(GTK_BOX(main_ctrns.bx_options), main_wnd.ed_bytes, FALSE, TRUE, 5);
  gtk_widget_set_usize(main_wnd.ed_bytes, 65, 0);
  gtk_widget_show(main_wnd.ed_bytes);

}

void create_win(int n_op, int n_number)
{
  /* options for rule creation. */
  ins_add_op = n_op;
  rule_number = n_number;

  /* lot of Gtk. */
  main_wnd.create_wnd = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (main_wnd.create_wnd), "Create Rule");
  gtk_window_set_policy(GTK_WINDOW (main_wnd.create_wnd), FALSE, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (main_wnd.create_wnd), 10);
  gtk_signal_connect(GTK_OBJECT (main_wnd.create_wnd), "delete_event",
		     GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);

  /* lot of controls containers */
  main_ctrns.bx_main = gtk_vbox_new(FALSE, 7);
  main_ctrns.bx_tables_chains = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_protocols = gtk_vbox_new(FALSE, 40);
  main_ctrns.bx_src_opts = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_dest_opts = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_actions = gtk_vbox_new(FALSE, 0);
  main_ctrns.bx_win_cmds = gtk_hbox_new(FALSE, 15);
  main_ctrns.bx_interfaces = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_options = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_protocols_aux = gtk_hbox_new(FALSE, 0);
  main_ctrns.bx_jumps = gtk_hbox_new(FALSE, 0);

  main_ctrns.frm_chain_sel = gtk_frame_new ("Chain Selection");
  main_ctrns.frm_protocols = gtk_frame_new ("Protocols"); 
  main_ctrns.frm_src_addr_port = gtk_frame_new ("Source Address and Port");
  main_ctrns.frm_dest_addr_port = gtk_frame_new ("Destination Address and Port");
  main_ctrns.frm_actions = gtk_frame_new ("Actions");
  main_ctrns.frm_inter = gtk_frame_new ("Interfaces");
  main_ctrns.frm_options = gtk_frame_new ("Options");
  
  gtk_container_add(GTK_CONTAINER (main_wnd.create_wnd), main_ctrns.bx_main);

  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.frm_chain_sel, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.frm_protocols, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), 
		     main_ctrns.frm_src_addr_port, FALSE, FALSE, 0); 
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), 
		     main_ctrns.frm_dest_addr_port, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.frm_inter, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.frm_options, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.frm_actions, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_main), main_ctrns.bx_win_cmds, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_protocols), 
		     main_ctrns.bx_protocols_aux, FALSE, FALSE, 5);
  gtk_box_pack_start(GTK_BOX (main_ctrns.bx_actions), main_ctrns.bx_jumps, FALSE, FALSE, 5);

  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_chain_sel), main_ctrns.bx_tables_chains);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_protocols), main_ctrns.bx_protocols);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_src_addr_port), main_ctrns.bx_src_opts);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_dest_addr_port), main_ctrns.bx_dest_opts);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_actions), main_ctrns.bx_actions);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_inter), main_ctrns.bx_interfaces);
  gtk_container_add(GTK_CONTAINER (main_ctrns.frm_options), main_ctrns.bx_options);

  gtk_widget_set_usize(main_ctrns.frm_chain_sel, 500, 50);
  gtk_widget_set_usize(main_ctrns.frm_src_addr_port, 500, 50);
  gtk_widget_set_usize(main_ctrns.frm_dest_addr_port, 500, 50);
  gtk_widget_set_usize(main_ctrns.frm_inter, 500, 50);
  gtk_widget_set_usize(main_ctrns.frm_options, 500, 50);

  gtk_widget_show(main_ctrns.frm_chain_sel);
  gtk_widget_show(main_ctrns.frm_protocols);
  gtk_widget_show(main_ctrns.frm_src_addr_port);
  gtk_widget_show(main_ctrns.frm_dest_addr_port);
  gtk_widget_show(main_ctrns.frm_actions);
  gtk_widget_show(main_ctrns.frm_inter);
  gtk_widget_show(main_ctrns.frm_options);
  
  create_cb_tables();
  create_cb_chains();
  create_cb_protocol();
  create_bt_tcp();
  create_bt_icmp();
  create_src_address_ed();
  create_src_port_ed();  
  create_dest_address_ed();
  create_dest_port_ed(); 
  create_chk_in_inter();
  create_chk_out_inter();
  create_options_chks();
  create_cb_jump();
  create_bt_set_reject();
  create_bt_log_rule();
  create_bt_cancel(); 
  create_bt_accept();

  gtk_widget_show(main_ctrns.bx_tables_chains);  
  gtk_widget_show(main_ctrns.bx_protocols);  
  gtk_widget_show(main_ctrns.bx_src_opts);
  gtk_widget_show(main_ctrns.bx_dest_opts);
  gtk_widget_show(main_ctrns.bx_actions);
  gtk_widget_show(main_ctrns.bx_win_cmds);
  gtk_widget_show(main_ctrns.bx_interfaces);
  gtk_widget_show(main_ctrns.bx_options);
  gtk_widget_show(main_ctrns.bx_protocols_aux);
  gtk_widget_show(main_ctrns.bx_jumps);
  gtk_widget_show(main_ctrns.bx_main);  
  gtk_widget_show(main_wnd.create_wnd);
}

/* in case is needed */
void on_cb_tables_changed(GtkWidget *widget, gpointer data)
{
  
}

/* in case is needed */
void on_cb_chains_changed(GtkWidget *widget, gpointer data)
{
  
}

/* in case is needed */
void on_cb_protocol_changed(GtkWidget *widget, gpointer data)
{
  gchar sz_tmp_buffer[64];

  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_protocol)->entry)));

  if(strcmp(sz_tmp_buffer, "tcp") == 0)
    {
      gtk_widget_set_sensitive(main_wnd.bt_icmp, FALSE);
      gtk_widget_set_sensitive(main_wnd.bt_tcp, TRUE);
    }
  else if(strcmp(sz_tmp_buffer, "icmp") == 0)
    {
      gtk_widget_set_sensitive(main_wnd.bt_icmp, TRUE);
      gtk_widget_set_sensitive(main_wnd.bt_tcp, FALSE);
    }
  else
    {
      gtk_widget_set_sensitive(main_wnd.bt_icmp, FALSE);
      gtk_widget_set_sensitive(main_wnd.bt_tcp, FALSE);
    }
}

void on_chk_fragment_clicked(GtkWidget *widget, gpointer data)
{
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(main_wnd.chk_fragment)))
    gtk_widget_set_sensitive(main_wnd.chk_frag_w_options, FALSE);
  else
    gtk_widget_set_sensitive(main_wnd.chk_frag_w_options, TRUE);
}

void on_chk_frag_w_options_clicked(GtkWidget *widget, gpointer data)
{
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(main_wnd.chk_frag_w_options)))
    gtk_widget_set_sensitive(main_wnd.chk_fragment, FALSE);
  else
    gtk_widget_set_sensitive(main_wnd.chk_fragment, TRUE);
}

void on_bt_tcp_clicked(GtkWidget *widget, gpointer data)
{
  set_tcp_win();
}

void on_bt_icmp_clicked(GtkWidget *widget, gpointer data)
{
  set_icmp_win();
}

/* just in case is needed */
void sport_callback(GtkWidget *widget, gpointer data)
{

}

/* just in case is needed */
void saddr_callback(GtkWidget *widget, gpointer data)
{
  
}

/* just in case is needed */
void daddr_callback(GtkWidget *widget, gpointer data)
{
  
}

/* just in case is needed */
void dport_callback(GtkWidget *widget, gpointer data)
{
  
}

void jump_callback(GtkWidget *widget, gpointer data)
{
  gchar sz_tmp_buffer[64];

  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_jump)->entry)));

  if(strcmp(sz_tmp_buffer, "REJECT") != 0)
    gtk_widget_set_sensitive(main_wnd.bt_set_reject, FALSE);
  else
    gtk_widget_set_sensitive(main_wnd.bt_set_reject, TRUE);
    
}

/* just in case is needed */
void ininter_callback(GtkWidget *widget, gpointer data)
{
  
}

/* in case is needed */
void outinter_callback(GtkWidget *widget, gpointer data)
{
  
}

/* just in case is needed */
void pkts_func(GtkWidget *widget, gpointer data)
{
  
}

/* just in case is needed */
void bytes_func(GtkWidget *widget, gpointer data)
{
  
}

void on_bt_accept_clicked(GtkWidget *widget, gpointer data)
{
  if (make_command_line() == 1)
    {
      update_rules();
      gtk_widget_destroy(main_wnd.create_wnd);
    }
  else
    {
      message_box(GTK_WINDOW(main_wnd.create_wnd), 
		  "There was an error creating the rule. Check fields pliz!",
		  GTK_MESSAGE_WARNING, GTK_BUTTONS_OK);
    }
}

/* this function creates the add or insert command and executes it */
int make_command_line()
{
  gchar command_line[BIG_STRING];
  gchar command_line_with_log[BIG_STRING];
  gchar chain_buf[32];
  gchar table_buf[32];
  gchar sz_tmp_buffer[64];
  gint i;

  /* initialize */
  memset(command_line, 0, sizeof(command_line));
  memset(command_line_with_log, 0, sizeof(command_line_with_log));
  memset(sz_tmp_buffer, 0, sizeof(sz_tmp_buffer));
  
  
  /* set table */
  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_tables)->entry)));

  strcat(command_line, "iptables -t ");
  strcat(command_line, sz_tmp_buffer);
  
  /* set chain */
  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_chains)->entry)));

  /* means wrong chain selected */
  if ((strncmp(sz_tmp_buffer, "-", 1)) == 0) 
    {
      return -1;
    }
  else
    {
      if (ins_add_op == 1) 
	{
	  strcat(command_line, " -I ");
	  strcat(command_line, sz_tmp_buffer);
	  sprintf(sz_tmp_buffer, " %i ", rule_number);
	  strcat(command_line, sz_tmp_buffer);
	}
      else if (ins_add_op == 2)
	{
	  strcat(command_line, " -A ");
	  strcat(command_line, sz_tmp_buffer);
	  strcat(command_line, " ");
	}
    }
  
  /* set fragment options */
  if (GTK_TOGGLE_BUTTON(main_wnd.chk_frag_w_options)->active)
    {
      strcat(command_line, "! -f ");
    }

  if (GTK_TOGGLE_BUTTON(main_wnd.chk_fragment)->active)
    {
      strcat(command_line, "-f ");
    }

  /* set counters */
  if (GTK_TOGGLE_BUTTON(main_wnd.chk_set_counters)->active)
    {
      strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_packages)));

      /* means packages edit is empty */
      if (strcmp(sz_tmp_buffer, "") == 0)
	return -1;
      
      strcat(command_line, "-c ");
      strcat(command_line, sz_tmp_buffer);

      strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_bytes)));

      /* means bytes edit is empty */
      if (strcmp(sz_tmp_buffer, "") == 0)
	return -1;
      
      strcat(command_line, " ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }

  /* set interfaces */
  if (GTK_TOGGLE_BUTTON(main_wnd.chk_in_inter)->active)
    {
      strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_in_inter)));

      /* no interface was supplied */
      if (strcmp(sz_tmp_buffer, "") == 0)
	return -1;
      
      strcat(command_line, "-i ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }
 
  if (GTK_TOGGLE_BUTTON(main_wnd.chk_out_inter)->active)
    {
      strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_out_inter)));

      /* no interface was supplied */
      if (strcmp(sz_tmp_buffer, "") == 0)
	return -1;
      
      strcat(command_line, "-o ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }
    
  /* set source address */
  strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_src_address)));

  if ((strcmp(sz_tmp_buffer, "")) != 0) 
    { 
      strcat(command_line, "-s ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");	
    }

  /* set destination address */
  strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_dest_address)));

  if ((strcmp(sz_tmp_buffer, "")) != 0) 
    {
      strcat(command_line, "-d ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }
  
  /* set protocol */
  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_protocol)->entry)));

  if (strcmp(sz_tmp_buffer, "") != 0)
    {
      strcat(command_line, "-p ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }
  else
    {
      strcat(command_line, "-p tcp ");
    }    
  
  /* tcp options */
  if ((strcmp(settcp_buf, "")) != 0)
    {
      strcat(command_line, " ");
      strcat(command_line, settcp_buf);
    }

  /* set destination port */
  strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_dest_port)));

  if (strcmp(sz_tmp_buffer, "") != 0) 
    {
      strcat(command_line, "--dport ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");	
    }

  /* set source port */
  strcpy(sz_tmp_buffer, gtk_entry_get_text(GTK_ENTRY(main_wnd.ed_src_port)));

  if ((strcmp(sz_tmp_buffer, "")) != 0) 
    {
      strcat(command_line, "--sport ");
      strcat(command_line, sz_tmp_buffer);
      strcat(command_line, " ");
    }

  /* set icmp */
  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_protocol)->entry)));

  if ((strcmp(sz_tmp_buffer, "icmp")) == 0) 
    {
      if ((strcmp(seticmp_buf, "")) != 0)
	strcat(command_line, seticmp_buf);
    }

  /* check if we have to log the rule */
  if (strcmp(logrule_buf, "") != 0)
    {
      strcpy(command_line_with_log, command_line);
      strcat(command_line_with_log, logrule_buf);
    }

  /* set jump */
  strcpy(sz_tmp_buffer, 
	 gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_jump)->entry)));
 
  strcat(command_line, "-j ");
  strcat(command_line, sz_tmp_buffer);
  strcat(command_line, " ");
  if ((strcmp(sz_tmp_buffer, "REJECT")) == 0)
    strcat(command_line, setreject_buf);
   
  if (ins_add_op == 1)
    {
      system(command_line); 
      system(command_line_with_log);
    }
  else 
    {
      system(command_line_with_log);
      system(command_line); 
    }
 
  memset(command_line, 0, sizeof(command_line));
  strcpy(seticmp_buf, "");
  strcpy(setreject_buf, "");
  strcpy(settcp_buf, "");
  strcpy(logrule_buf, "");

  return 1;
}
