/*
 * Gtk-IPtables is Copyright (C) 2006  Daniel E. Testa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "main.h"

/* 
 * Global data - help for using iptables. 
 * variables to get table, chain and rule for deletion.
 */
typedef struct _help_gtkipt
{
  gchar s_selected_table[32]; /* filter */
  gchar s_selected_chain[32]; /* INPUT */
  gint rule_number;
  gint del_flag;
  gint ins_add_op;
} HELP_GTKIPT;

HELP_GTKIPT gbl_gtkipt;

/* Widgets used by the main window */
typedef struct _st_wnd
{
  GtkWidget *wnd_main;
  GtkWidget *bt_insert;
  GtkWidget *bt_add;
  GtkWidget *bt_del;
  GtkWidget *bt_exit;
  GtkWidget *bt_about;
  GtkWidget *frm_main;
  GtkWidget *bx_one;
  GtkWidget *bx_two;
  GtkWidget *bx_three;
  GtkWidget *cb_tables;
  GtkWidget *ls_rules;
  GtkWidget *scrolled_wnd_list;
} STWND;

static STWND main_wnd;

static void initialize_global()
{
  strcpy(gbl_gtkipt.s_selected_table, "filter");
  strcpy(gbl_gtkipt.s_selected_chain, "INPUT");
  gbl_gtkipt.del_flag = -1;
  gbl_gtkipt.ins_add_op = -1;
}

static int want_to_exit()
{
  int n_res;

  /* ask first */
  n_res = message_box(GTK_WINDOW(main_wnd.wnd_main), "Would you like to exit?",
		      GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO);

  if (n_res != GTK_RESPONSE_YES)
    return FALSE;

  return TRUE;
}

static void create_insert_button()
{
  GtkWidget *img_label;

  main_wnd.bt_insert = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_insert), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_insert_clicked), NULL);   
  
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_insert.png", " Insert ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_insert), img_label);
  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.bt_insert, 
		     FALSE, TRUE, 0);
  gtk_widget_show (main_wnd.bt_insert);
}

static void create_add_button()
{
  GtkWidget *img_label;

  main_wnd.bt_add = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_add), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_add_clicked), NULL);
  
  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.bt_add, 
		     FALSE, TRUE , 0);
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_newwin.png", " Create ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_add), img_label);
  gtk_widget_show (main_wnd.bt_add);
}

static void create_delete_button()
{
  GtkWidget *img_label;

  main_wnd.bt_del = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_del), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_del_clicked), NULL);
  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.bt_del, 
		     FALSE, TRUE, 0);
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_trash.png", " Delete ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_del), img_label);
  gtk_widget_show (main_wnd.bt_del);
}

static void create_exit_button()
{
  GtkWidget *img_label;

  main_wnd.bt_exit = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_exit), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_exit_clicked), NULL);

  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.bt_exit, 
		     FALSE, TRUE, 0);
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_exit.png", " Exit ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_exit), img_label);
  gtk_widget_show (main_wnd.bt_exit);
}

static void create_about_button()
{
  GtkWidget *img_label;

  main_wnd.bt_about = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT (main_wnd.bt_about), "clicked",
		     GTK_SIGNAL_FUNC (on_bt_about_clicked), NULL);

  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.bt_about, 
		     FALSE, TRUE, 0);
  img_label = png_label_box(PIXMAPS_DIR"/gtkipt_about.png", " About ");
  gtk_widget_show(img_label);
  gtk_container_add(GTK_CONTAINER (main_wnd.bt_about), img_label);
  gtk_widget_show(main_wnd.bt_about);
}

static void create_rules_list()
{
  gchar *a_columns_titles[LIST_COLS] = {"Num", "Pkts", "Bytes" ,"Target", 
					"Prot", "Opt", "In", "Out", "Source", 
					"Destination", "Extra Info"};
  
  main_wnd.scrolled_wnd_list = gtk_scrolled_window_new( NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(main_wnd.scrolled_wnd_list),
				 GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
  gtk_widget_set_usize(main_wnd.scrolled_wnd_list, 700, 300);
  gtk_box_pack_start(GTK_BOX(main_wnd.bx_three), main_wnd.scrolled_wnd_list, 
		     TRUE, TRUE, 0);
  gtk_widget_show(main_wnd.scrolled_wnd_list);

  main_wnd.ls_rules = gtk_clist_new_with_titles(LIST_COLS, a_columns_titles);
  gtk_container_add(GTK_CONTAINER(main_wnd.scrolled_wnd_list), 
		    main_wnd.ls_rules);
  gtk_signal_connect(GTK_OBJECT (main_wnd.ls_rules), "select_row",
		     GTK_SIGNAL_FUNC(on_ls_rules_select_row), NULL); 
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 0, 35);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 1, 40);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 2, 60);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 3, 65);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 4, 40);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 5, 40);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 6, 70);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 7, 70);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 8, 175);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 9, 175);
  gtk_clist_set_column_width(GTK_CLIST(main_wnd.ls_rules), 10, 270);
  
  gtk_widget_show(main_wnd.ls_rules);
}

static void create_tables_combo()     
{
  GList *gl_tables = NULL;
 
  gl_tables = g_list_append(gl_tables, FILTER_INPUT);
  gl_tables = g_list_append(gl_tables, FILTER_FORWARD);
  gl_tables = g_list_append(gl_tables, FILTER_OUTPUT);
  
  gl_tables = g_list_append(gl_tables, NAT_PREROUTING);
  gl_tables = g_list_append(gl_tables, NAT_POSTROUTING);
  gl_tables = g_list_append(gl_tables, NAT_OUTPUT);
  
  gl_tables = g_list_append(gl_tables, MANGLE_PREROUTING);
  gl_tables = g_list_append(gl_tables, MANGLE_OUTPUT);
  gl_tables = g_list_append(gl_tables, MANGLE_INPUT);
  gl_tables = g_list_append(gl_tables, MANGLE_FORWARD);
  gl_tables = g_list_append(gl_tables, MANGLE_POSTROUTING);

  main_wnd.cb_tables = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(main_wnd.cb_tables), gl_tables);
  gtk_combo_set_value_in_list(GTK_COMBO(main_wnd.cb_tables), TRUE, FALSE);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(main_wnd.cb_tables)->entry), "changed",
		     GTK_SIGNAL_FUNC (on_cb_tables_changed), NULL);
  gtk_box_pack_start(GTK_BOX (main_wnd.bx_two), main_wnd.cb_tables, FALSE, TRUE, 0);
  gtk_widget_show(main_wnd.cb_tables);
}

static void create_boxes()
{
  main_wnd.bx_one = gtk_vbox_new(FALSE, 15);
  main_wnd.bx_two = gtk_hbox_new(FALSE, 10);
  main_wnd.bx_three = gtk_hbox_new(FALSE, 0);
}

int main(int argc, char *argv[])
{
  gtk_init(&argc, &argv);
	
  initialize_global();

  main_wnd.wnd_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(main_wnd.wnd_main), STR_APP_TITLE);
  gtk_window_set_policy(GTK_WINDOW(main_wnd.wnd_main), FALSE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT (main_wnd.wnd_main), "delete_event",
		      GTK_SIGNAL_FUNC (on_delete_wnd_event), NULL);
  gtk_container_set_border_width (GTK_CONTAINER (main_wnd.wnd_main), 10);

  if(!have_root_access())
    {
      message_box(GTK_WINDOW(main_wnd.wnd_main),
		  "You are not 'root'.\n"
		  "You must be 'root' in order to execute 'iptables'.",
		  GTK_MESSAGE_INFO, GTK_BUTTONS_OK);

      exit(-1);
    }

  create_boxes();
  
  main_wnd.frm_main = gtk_frame_new(" netfilter.org // iptables ");

  gtk_container_add(GTK_CONTAINER (main_wnd.wnd_main), main_wnd.bx_one);
  gtk_box_pack_end(GTK_BOX (main_wnd.bx_one), main_wnd.bx_two, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (main_wnd.bx_one), main_wnd.frm_main, TRUE, TRUE, 0);
  gtk_container_add(GTK_CONTAINER(main_wnd.frm_main), main_wnd.bx_three);
  gtk_widget_set_usize(main_wnd.frm_main, 706, 300);
  gtk_widget_show(main_wnd.frm_main);
	
  create_rules_list();
  create_tables_combo();
  create_insert_button();
  create_add_button();
  create_delete_button();
  create_exit_button();
  create_about_button();
	
  update_rules();

  gtk_widget_show(main_wnd.bx_two);
  gtk_widget_show(main_wnd.bx_three);
  gtk_widget_show (main_wnd.bx_one);  
  gtk_widget_show (main_wnd.wnd_main);

  gtk_main();

  return 0;
}

/* some functions */
void update_rules()
{
  get_rules(main_wnd.ls_rules);
}

/* 
 * this function executes the command for iptables
 * so we can get output and put it on the list.
 */
void get_rules(GtkWidget *pls_rules)
{
  int n_cell = 0;
  int n_row = 0;
  int n_index = 0;
  char ch_tmp;
  char s_str_buffer[64];
  char s_popen_cmd[256];
  gchar *s_row_cells[12] = { NULL, NULL, NULL, NULL, NULL, NULL,
			     NULL, NULL, NULL, NULL, NULL};
  gchar *p_command = NULL;
  FILE *p_cmd = NULL;

  memset(s_popen_cmd, 0, sizeof(s_popen_cmd));

  /* Let's check that iptables is in the path */
  p_command = g_find_program_in_path("iptables");

  /* do something more useful */
  if (p_command == NULL)
    {
      message_box(GTK_WINDOW(main_wnd.wnd_main),
		  "iptables was not found in path!",
		  GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
      exit(-1);
    }

  /* we use the path returned by g_find_program_in_path */
  strcpy(s_popen_cmd, p_command);
  strcat(s_popen_cmd, " -t ");

  /* free resource */
  g_free(p_command);
	
  /* build command */
  strcat(s_popen_cmd, gbl_gtkipt.s_selected_table);
  strcat(s_popen_cmd, " -L ");
  strcat(s_popen_cmd, gbl_gtkipt.s_selected_chain);
  strcat(s_popen_cmd, " --line-numbers -v");

  p_cmd = popen(s_popen_cmd, "r");
  
  /* do something more useful */
  if (p_cmd == NULL)
    exit(-1);
	
  gtk_clist_clear((GtkCList *) pls_rules);
	
  /* omit the first 2 lines */
  bypass_lines(p_cmd, 2);
	
  while (feof(p_cmd) == 0)
    {			
      fscanf(p_cmd, "%s", s_str_buffer);
      if (feof(p_cmd) != 0)
	break;
		
      if (n_cell == 0)
	gtk_clist_insert((GtkCList *) pls_rules, n_row + 1, s_row_cells);
			
      gtk_clist_set_text(GTK_CLIST (pls_rules), n_row, n_cell, s_str_buffer);
				
      n_cell++;
		
      if (n_cell == 10) 
	{
	  memset(s_str_buffer, 0, sizeof(s_str_buffer));
	  while ((ch_tmp = fgetc(p_cmd)) != ASCII_LF)
	    {
	      s_str_buffer[n_index] = ch_tmp;
	      n_index++;
	    }
	  s_str_buffer[n_index] = ASCII_NUL;
	  gtk_clist_set_text(GTK_CLIST (pls_rules), n_row, n_cell, 
			     left_trim(s_str_buffer));				
	  n_index = 0;
	  n_cell = 0;
	  n_row++;
	}
    }
  pclose(p_cmd);
}

/* moves the file descriptor the specified amount of lines by n_lines */
void bypass_lines(FILE *p_file, int n_lines)
{
  char ch_aux;
  int n_line_counter = 0;
	
  while (n_line_counter < n_lines)
    {
      ch_aux = getc(p_file);
      if (ch_aux == ASCII_LF)
	n_line_counter++;
    }
}

/* Callbacks */
gint on_delete_wnd_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  if(want_to_exit() == TRUE)
    {
      gtk_main_quit();
      return(FALSE);
    }

  return(TRUE);
}

void on_bt_exit_clicked(GtkWidget *widget, gpointer data)
{
  if(want_to_exit() == TRUE)
      gtk_main_quit();
}

void on_bt_about_clicked(GtkWidget *widget, gpointer data)
{
  message_box(GTK_WINDOW(main_wnd.wnd_main), 
	      STR_APP_TITLE"\n"
	      "email: danieltes@gmail.com\n"
	      "web: gtk-iptables.sourceforge.net\n"
	      "(c) 2006 Daniel E. Testa\n",
	      GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
}

void on_cb_tables_changed(GtkWidget *widget, gpointer data)
{
  gchar *s_cb_text = (gchar*)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_wnd.cb_tables)->entry));
    
  if ((strcmp(FILTER_INPUT, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "filter");
      strcpy(gbl_gtkipt.s_selected_chain, "INPUT");
    }
  else if ((strcmp(FILTER_FORWARD, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "filter");
      strcpy(gbl_gtkipt.s_selected_chain, "FORWARD");
    }
  else if ((strcmp(FILTER_OUTPUT, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "filter");
      strcpy(gbl_gtkipt.s_selected_chain, "OUTPUT");
    }
  else if ((strcmp(NAT_PREROUTING, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "nat");
      strcpy(gbl_gtkipt.s_selected_chain, "PREROUTING");
    }
  else if ((strcmp(NAT_POSTROUTING, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "nat");
      strcpy(gbl_gtkipt.s_selected_chain, "POSTROUTING");
    }
  else if ((strcmp(NAT_OUTPUT, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "nat");
      strcpy(gbl_gtkipt.s_selected_chain, "OUTPUT");
    }
  else if ((strcmp(MANGLE_INPUT, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "mangle");
      strcpy(gbl_gtkipt.s_selected_chain, "INPUT");
    }
  else if ((strcmp(MANGLE_FORWARD, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "mangle");
      strcpy(gbl_gtkipt.s_selected_chain, "FORWARD");
    }
  else if ((strcmp(MANGLE_OUTPUT, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "mangle");
      strcpy(gbl_gtkipt.s_selected_chain, "OUTPUT");
    }
  else if ((strcmp(MANGLE_PREROUTING, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "mangle");
      strcpy(gbl_gtkipt.s_selected_chain, "PREROUTING");
    }
  else if ((strcmp(MANGLE_POSTROUTING, s_cb_text)) == 0)
    {
      strcpy(gbl_gtkipt.s_selected_table, "mangle");
      strcpy(gbl_gtkipt.s_selected_chain, "POSTROUTING");
    }
  else
    {
      /* TODO */
    }
		
  update_rules();
}

/* just in case is needed */
void on_ls_rules_select_row(GtkWidget *widget, gint row, gint column,
			 GdkEventButton *event, gpointer data)
{
  
}

void on_bt_del_clicked(GtkWidget *widget, gpointer data)
{
  gchar command_line[128] = "iptables ";
  gchar buf[5];
  int n_msg_response;
  int n_selected_index = -1;

  n_selected_index = get_list_selected_row(main_wnd.ls_rules);

  /* check that something this selected first */
  if (n_selected_index == -1)
    {
      message_box(GTK_WINDOW(main_wnd.wnd_main), "There's no rule selected!",
		  GTK_MESSAGE_WARNING, GTK_BUTTONS_OK);
    
      return;
    }
  
  /* ask first */
  n_msg_response = message_box(GTK_WINDOW(main_wnd.wnd_main), 
			      "You are about to delete a rule. Continue?",
			      GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO);

  if (n_msg_response != GTK_RESPONSE_YES)
    return;
  
  /* remove from control */
  gtk_clist_remove(GTK_CLIST(main_wnd.ls_rules), n_selected_index);
	
  /* remove from iptables */
  strcat(command_line, "-t ");
  strcat(command_line, gbl_gtkipt.s_selected_table);
  strcat(command_line, " ");
  strcat(command_line, "-D ");
  strcat(command_line, gbl_gtkipt.s_selected_chain);
  strcat(command_line, " ");
  sprintf(buf, "%i", n_selected_index + 1);
  strcat(command_line, buf);

  /* execute command */
  system(command_line);

  /* update list */
  update_rules();
}

void on_bt_insert_clicked(GtkWidget *widget, gpointer data)
{
  int n_selected_index = get_list_selected_row(main_wnd.ls_rules);

  /* check that something is selected first */
  if (n_selected_index == -1)
    {
      message_box(GTK_WINDOW(main_wnd.wnd_main), "There's no position selected!",
		  GTK_MESSAGE_WARNING, GTK_BUTTONS_OK);
    
      return;
    }
  
  gbl_gtkipt.ins_add_op = 1;
  create_win(CREATE_INSERT, n_selected_index + 1);
}

void on_bt_add_clicked(GtkWidget *widget, gpointer data)
{
  gbl_gtkipt.ins_add_op = 2;
  create_win(CREATE_ADD, -1);
}
